import React, { useRef, useState, useEffect } from "react";
import {
  StyleSheet,
  Text,
  View,
  SafeAreaView,
  FlatList,
  Animated,
  Dimensions,
  TouchableWithoutFeedback,
  ImageBackground,
  Image,
} from "react-native";
const windowWidth = Dimensions.get("window").width;
const windowHeight = Dimensions.get("window").height;
import PropTypes from "prop-types";
import helloImage from "./images/avatar-4.jpg";

const sliderData = [
  "https://img.freepik.com/free-vector/mineral-water-bottle-ad-banner-flask-with-drink_107791-2565.jpg?size=626&ext=jpg",
  "https://2.bp.blogspot.com/-7u_-jGO8YiU/VUxg1PTVyeI/AAAAAAAAMK0/yC6I-qSzLnM/s1600/mineral%2Bwater%2Bcans%2Bwith%2Bwater%2Beffect.jpg",
  "https://cdn.vox-cdn.com/thumbor/5o2nkGAEfQFPpjfKEocJlLyiOrc=/0x0:4500x3000/1200x800/filters:focal(1890x1140:2610x1860)/cdn.vox-cdn.com/uploads/chorus_image/image/53047641/512015968.0.jpg",
];

const AnimatedImageSlider = ({
  activeDotColor,
  inActiveDotColor,
  imageBorderRadius,
  imageWidth,
  imageHeight,
  autoSlideDuration,
  dotsContainerStyle,
  imageContainerStyle,
  data,
}) => {
  const sliderScrollX = useRef(new Animated.Value(0)).current;

  const flatListRef = useRef();

  const [count, setCount] = useState(0);

  useEffect(() => {
    if (data) {
      const timeout = setTimeout(async () => {
        if (count == data?.length) {
          flatListRef.current.scrollToIndex({
            index: 0,
            animated: true,
          });
          return setCount(0);
        }

        setCount(count + 1);
        // console.log('COUNTER ', counterTest);
        flatListRef.current.scrollToIndex({
          index: count,
          animated: true,
        });
      }, autoSlideDuration);

      return () => {
        clearTimeout(timeout);
      };
    }
  }, [count]);

  function renderSlider() {
    return (
      <Animated.FlatList
        ref={flatListRef}
        horizontal
        pagingEnabled
        snapToAlignment="center"
        snapToInterval={windowWidth}
        showsHorizontalScrollIndicator={false}
        scrollEventThrottle={16}
        decelerationRate={0}
        contentContainerStyle={{
          marginTop: 24,
        }}
        data={data}
        keyExtractor={(item, index) => `${index}`}
        onScroll={Animated.event(
          [{ nativeEvent: { contentOffset: { x: sliderScrollX } } }],
          { useNativeDriver: false }
        )}
        onMomentumScrollBegin={(event) => {
          let sliderIndex = event.nativeEvent.contentOffset.x
            ? event.nativeEvent.contentOffset.x / windowWidth
            : 0;

          setCount(Math.round(sliderIndex));
        }}
        renderItem={({ item, index }) => {
          return (
            <TouchableWithoutFeedback
              onPress={() => console.log("SLIDER PRESSED")}
            >
              <View
                style={{
                  width: windowWidth,
                  alignItems: "center",
                  justifyContent: "center",
                  ...imageContainerStyle,
                }}
              >
                {/* Thumbnail */}
                <ImageBackground
                  source={typeof item == "number" ? item : { uri: item }}
                  resizeMode="cover"
                  style={{
                    width: imageWidth,
                    height: imageHeight,
                  }}
                  imageStyle={{
                    borderRadius: imageBorderRadius,
                  }}
                ></ImageBackground>
              </View>
            </TouchableWithoutFeedback>
          );
        }}
      />
    );
  }

  function renderDots() {
    const dotPosition = Animated.divide(sliderScrollX, windowWidth);

    return (
      <View
        style={{
          marginTop: 24,
          flexDirection: "row",
          alignItems: "center",
          justifyContent: "center",
          ...dotsContainerStyle,
        }}
      >
        {data?.map((item, index) => {
          const opacity = dotPosition.interpolate({
            inputRange: [index - 1, index, index + 1],
            outputRange: [0.3, 1, 0.3],
            extrapolate: "clamp",
          });

          const dotWidth = dotPosition.interpolate({
            inputRange: [index - 1, index, index + 1],
            outputRange: [6, 20, 6],
            extrapolate: "clamp",
          });

          const dotColor = dotPosition.interpolate({
            inputRange: [index - 1, index, index + 1],
            outputRange: [inActiveDotColor, activeDotColor, inActiveDotColor],
            extrapolate: "clamp",
          });

          return (
            <Animated.View
              opacity={opacity}
              key={`dot-${index}`}
              style={{
                borderRadius: 24,
                marginHorizontal: 3,
                width: dotWidth,
                height: 6,
                backgroundColor: dotColor,
              }}
            ></Animated.View>
          );
        })}
      </View>
    );
  }

  if (data?.length) {
    return (
      <SafeAreaView>
        {renderSlider()}
        {renderDots()}
        {/* {data?.length ? (
          <Text>Data Available</Text>
        ) : (
          <Text>Data Not Available</Text>
        )} */}
      </SafeAreaView>
    );
  } else {
    return (
      <SafeAreaView>
        <Text>Data not Available</Text>
      </SafeAreaView>
    );
  }
};

AnimatedImageSlider.propTypes = {
  activeDotColor: PropTypes.string,
  inActiveDotColor: PropTypes.string,
  imageBorderRadius: PropTypes.number,
  imageWidth: PropTypes.number || PropTypes.string,
  imageHeight: PropTypes.number || PropTypes.string,
  autoSlideDuration: PropTypes.number,
  dotsContainerStyle: PropTypes.object,
  imageContainerStyle: PropTypes.object,
};

// Same approach for defaultProps too
AnimatedImageSlider.defaultProps = {
  activeDotColor: "#0073e6",
  inActiveDotColor: "gray",
  imageBorderRadius: 40,
  imageWidth: windowWidth * 0.85,
  imageHeight: windowWidth * 0.85,
  autoSlideDuration: 3000,
  dotsContainerStyle: {},
  imageContainerStyle: {},
};

export default AnimatedImageSlider;

const styles = StyleSheet.create({});
